@@ -1,12 +1,26 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
- * e-table-click-to-add.c: A canvas item based view of the ETableColumn.
+/* 
+ * e-table-click-to-add.c
+ * Copyright 2000, 2001, Ximian, Inc.
  *
- * Author:
- *   Miguel de Icaza (miguel@gnu.org)
+ * Authors:
+ *   Chris Lahey <clahey@ximian.com>
  *
- * Copyright 1999, 2000 Ximian, Inc.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License, version 2, as published by the Free Software Foundation.
+ *
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ * 02111-1307, USA.
  */
+
 #include <config.h>
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtksignal.h>
@@ -257,14 +271,6 @@ etcta_unrealize (GnomeCanvasItem *item)
 		(*GNOME_CANVAS_ITEM_CLASS (etcta_parent_class)->unrealize)(item);
 }
 
-static double
-etcta_point (GnomeCanvasItem *item, double x, double y, int cx, int cy,
-	    GnomeCanvasItem **actual_item)
-{
-	*actual_item = item;
-	return 0.0;
-}
-
 static void finish_editing (ETableClickToAdd *etcta);
 
 static int
@@ -331,7 +337,7 @@ static int
 etcta_event (GnomeCanvasItem *item, GdkEvent *e)
 {
 	ETableClickToAdd *etcta = E_TABLE_CLICK_TO_ADD (item);
-	
+
 	switch (e->type){
 	case GDK_BUTTON_PRESS:
 		if (etcta->text) {
@@ -433,7 +439,6 @@ etcta_class_init (ETableClickToAddClass *klass)
 
 	item_class->realize     = etcta_realize;
 	item_class->unrealize   = etcta_unrealize;
-	item_class->point       = etcta_point;
 	item_class->event       = etcta_event;
 
 	gtk_object_add_arg_type ("ETableClickToAdd::header", GTK_TYPE_OBJECT,
