@@ -1,13 +1,27 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
- * E-table.c: A graphical view of a Table.
+/* 
+ * e-table.c - A graphical view of a Table.
+ * Copyright 1999, 2000, 2001, Ximian, Inc.
  *
- * Author:
- *   Miguel de Icaza (miguel@ximian.com)
+ * Authors:
  *   Chris Lahey <clahey@ximian.com>
+ *   Miguel de Icaza <miguel@ximian.com>
  *
- * Copyright 1999, 2000, 2001, Ximian, Inc
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License, version 2, as published by the Free Software Foundation.
+ *
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ * 02111-1307, USA.
  */
+
 #include <config.h>
 #include <stdlib.h>
 #include <stdio.h>
@@ -41,6 +55,14 @@
 
 #define PARENT_TYPE gtk_table_get_type ()
 
+#define d(x)
+
+#if d(!)0
+#define e_table_item_leave_edit_(x) (e_table_item_leave_edit((x)), g_print ("%s: e_table_item_leave_edit\n", __FUNCTION__))
+#else
+#define e_table_item_leave_edit_(x) (e_table_item_leave_edit((x)))
+#endif
+
 static GtkObjectClass *e_table_parent_class;
 
 enum {
@@ -530,7 +552,7 @@ group_start_drag (ETableGroup *etg, int row, int col, GdkEvent *event, ETable *e
 {
 	int return_val = 0;
 	gtk_signal_emit (GTK_OBJECT (et),
-			 et_signals [KEY_PRESS],
+			 et_signals [START_DRAG],
 			 row, col, event, &return_val);
 	return return_val;
 }
@@ -700,20 +722,84 @@ et_canvas_realize (GtkWidget *canvas, ETable *e_table)
 		NULL);
 }
 
-static gint
-et_canvas_button_press (GtkWidget *canvas, GdkEventButton *event, ETable *e_table)
+static void
+et_eti_leave_edit (ETable *et)
 {
+	GnomeCanvas *canvas = et->table_canvas;
 	if (GTK_WIDGET_HAS_FOCUS(canvas)) {
 		GnomeCanvasItem *item = GNOME_CANVAS(canvas)->focused_item;
 
 		if (E_IS_TABLE_ITEM(item)) {
-			e_table_item_leave_edit(E_TABLE_ITEM(item));
+			e_table_item_leave_edit_(E_TABLE_ITEM(item));
 		}
 	}
+}
+
+static gint
+et_canvas_root_event (GnomeCanvasItem *root, GdkEvent *event, ETable *e_table)
+{
+	switch (event->type) {
+	case GDK_BUTTON_PRESS:
+	case GDK_2BUTTON_PRESS:
+	case GDK_BUTTON_RELEASE:
+		if (event->button.button != 4 && event->button.button != 5) {
+			et_eti_leave_edit (e_table);
+			return TRUE;
+		}
+		break;
+	default:
+		break;
+	}
 
 	return FALSE;
 }
 
+/* Finds the first descendant of the group that is an ETableItem and focuses it */
+static void
+focus_first_etable_item (ETableGroup *group)
+{
+	GnomeCanvasGroup *cgroup;
+	GList *l;
+
+	cgroup = GNOME_CANVAS_GROUP (group);
+
+	for (l = cgroup->item_list; l; l = l->next) {
+		GnomeCanvasItem *i;
+
+		i = GNOME_CANVAS_ITEM (l->data);
+
+		if (E_IS_TABLE_GROUP (i))
+			focus_first_etable_item (E_TABLE_GROUP (i));
+		else if (E_IS_TABLE_ITEM (i)) {
+			e_table_item_set_cursor (E_TABLE_ITEM (i), 0, 0);
+			gnome_canvas_item_grab_focus (i);
+		}
+	}
+}
+
+/* Handler for focus events in the table_canvas; we have to repaint ourselves
+ * always, and also give the focus to some ETableItem if we get focused.
+ */
+static gint
+table_canvas_focus_event_cb (GtkWidget *widget, GdkEventFocus *event, gpointer data)
+{
+	GnomeCanvas *canvas;
+	ETable *etable;
+
+	gtk_widget_queue_draw (widget);
+
+	if (!event->in)
+		return TRUE;
+
+	canvas = GNOME_CANVAS (widget);
+	etable = E_TABLE (data);
+
+	if (!canvas->focused_item && etable->group)
+		focus_first_etable_item (etable->group);
+
+	return TRUE;
+}
+
 static void
 e_table_setup_table (ETable *e_table, ETableHeader *full_header, ETableHeader *header,
 		     ETableModel *model)
@@ -724,10 +810,10 @@ e_table_setup_table (ETable *e_table, ETableHeader *full_header, ETableHeader *h
 		GTK_SIGNAL_FUNC (table_canvas_size_allocate), e_table);
 	gtk_signal_connect (
 		GTK_OBJECT (e_table->table_canvas), "focus_in_event",
-		GTK_SIGNAL_FUNC (gtk_widget_queue_draw), e_table);
+		GTK_SIGNAL_FUNC (table_canvas_focus_event_cb), e_table);
 	gtk_signal_connect (
 		GTK_OBJECT (e_table->table_canvas), "focus_out_event",
-		GTK_SIGNAL_FUNC (gtk_widget_queue_draw), e_table);
+		GTK_SIGNAL_FUNC (table_canvas_focus_event_cb), e_table);
 
 	gtk_signal_connect (
 		GTK_OBJECT (e_table), "drag_begin",
@@ -774,8 +860,8 @@ e_table_setup_table (ETable *e_table, ETableHeader *full_header, ETableHeader *h
 		GTK_OBJECT(e_table->table_canvas), "realize",
 		GTK_SIGNAL_FUNC(et_canvas_realize), e_table);
 	gtk_signal_connect (
-		GTK_OBJECT(e_table->table_canvas), "button_press_event",
-		GTK_SIGNAL_FUNC(et_canvas_button_press), e_table);
+		GTK_OBJECT(gnome_canvas_root (e_table->table_canvas)), "event",
+		GTK_SIGNAL_FUNC(et_canvas_root_event), e_table);
 	e_table->canvas_vbox = gnome_canvas_item_new(
 		gnome_canvas_root(e_table->table_canvas),
 		e_canvas_vbox_get_type(),
@@ -833,6 +919,7 @@ e_table_set_state_object(ETable *e_table, ETableState *state)
 	}
 	if (state->sort_info) {
 		e_table->sort_info = e_table_sort_info_duplicate(state->sort_info);
+		e_table_sort_info_set_can_group (e_table->sort_info, e_table->allow_grouping);
 		e_table->group_info_change_id =
 			gtk_signal_connect (GTK_OBJECT (e_table->sort_info),
 					    "group_info_changed",
@@ -1024,9 +1111,11 @@ et_real_construct (ETable *e_table, ETableModel *etm, ETableExtras *ete,
 
 	e_table->header = e_table_state_to_header (GTK_WIDGET(e_table), e_table->full_header, state);
 	e_table->horizontal_scrolling = specification->horizontal_scrolling;
+	e_table->allow_grouping = specification->allow_grouping;
 
 	e_table->sort_info = state->sort_info;
 	gtk_object_ref (GTK_OBJECT (state->sort_info));
+	e_table_sort_info_set_can_group (e_table->sort_info, e_table->allow_grouping);
 
 	e_table->group_info_change_id =
 		gtk_signal_connect (GTK_OBJECT (e_table->sort_info), "group_info_changed",
@@ -1042,6 +1131,7 @@ et_real_construct (ETable *e_table, ETableModel *etm, ETableExtras *ete,
 	gtk_object_set (GTK_OBJECT (e_table->selection),
 			"model", etm,
 			"sorter", e_table->sorter,
+			"header", e_table->header,
 			NULL);
 
 	gtk_signal_connect(GTK_OBJECT(e_table->selection), "selection_changed",
@@ -1417,6 +1507,14 @@ e_table_right_click_up (ETable *table)
 	e_selection_model_right_click_up(E_SELECTION_MODEL(table->selection));
 }
 
+void
+e_table_commit_click_to_add (ETable *table)
+{
+	et_eti_leave_edit (table);
+	if (table->click_to_add)
+		e_table_click_to_add_commit(E_TABLE_CLICK_TO_ADD(table->click_to_add));
+}
+
 static void
 et_get_arg (GtkObject *o, GtkArg *arg, guint arg_id)
 {
@@ -1618,6 +1716,15 @@ e_table_get_cell_geometry (ETable *table,
 	}
 }
 
+ESelectionModel *
+e_table_get_selection_model (ETable *table)
+{
+	g_return_val_if_fail (table != NULL, NULL);
+	g_return_val_if_fail (E_IS_TABLE (table), NULL);
+
+	return E_SELECTION_MODEL (table->selection);
+}
+
 struct _ETableDragSourceSite
 {
 	GdkModifierType    start_button_mask;
